#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.RandomGenerator.SetSeed(0)

size = 200

# input sample
inputSample = ot.Uniform(-1.0, 1.0).getSample(size)
outputSample = ot.Sample(inputSample)

# Evaluation of y = ax + b (a: scale, b: translate)

# scale
scale = [3.0]
outputSample *= scale

# translate sample
translate = [3.1]
outputSample += translate

# Finally inverse transform using an arbitrary lambda
lamb = [1.8]
boxCoxFunction = ot.InverseBoxCoxEvaluation(lamb)

# transform y using BoxCox function
outputSample = boxCoxFunction(outputSample)

# Add small noise
epsilon = ot.Normal(0, 1.0e-2).getSample(size)
outputSample += epsilon

# Now we build the factory
factory = ot.BoxCoxFactory()

# Creation of the BoxCoxTransform
result = ot.GeneralLinearModelResult()
basis = ot.LinearBasisFactory(1).build()
covarianceModel = ot.DiracCovarianceModel()
shift = [1.0e-1]
myBoxCox = factory.build(inputSample, outputSample,
                         covarianceModel, basis, shift, result)

print("myBoxCox (GLM) =", myBoxCox)
print("GLM result     =", result)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
from math import *

ot.TESTPREAMBLE()


# Problem parameters
inputDimension = 2
outputDimension = 1
rho = 0.3
a = 4.
b = 5.

# Reference analytical values
covTh = a * a + b * b + 2 * a * b * rho
Si = [[(a * a + a * b * rho) / covTh, a * a / covTh],
      [(b * b + a * b * rho) / covTh, b * b / covTh]]

# Model
inputName = ["X1", "X2", "a", "b"]
formula = ["a * X1 + b * X2"]

full = ot.SymbolicFunction(inputName, formula)
model = ot.ParametricFunction(full, [2, 3], [a, b])

# Input distribution
distribution = ot.ComposedDistribution([ot.Normal()] * inputDimension)

# Correlated input distribution
S = ot.CorrelationMatrix(inputDimension)
S[1, 0] = 0.3
R = ot.NormalCopula().GetCorrelationFromSpearmanCorrelation(S)
myCopula = ot.NormalCopula(R)
myCorrelatedInputDistribution = ot.ComposedDistribution(
    [ot.Normal()] * inputDimension, myCopula)

sample = myCorrelatedInputDistribution.getSample(2000)

# Orthogonal basis
enumerateFunction = ot.LinearEnumerateFunction(inputDimension)
productBasis = ot.OrthogonalProductPolynomialFactory(
    [ot.HermiteFactory()] * inputDimension, enumerateFunction)
# Adaptive strategy
adaptiveStrategy = ot.FixedStrategy(
    productBasis, enumerateFunction.getStrataCumulatedCardinal(4))
# Projection strategy
samplingSize = 250
projectionStrategy = ot.LeastSquaresStrategy(
    ot.MonteCarloExperiment(samplingSize))

# Polynomial chaos algorithm
algo = ot.FunctionalChaosAlgorithm(
    model, distribution, adaptiveStrategy, projectionStrategy)
algo.run()

# Post-process the results
result = ot.FunctionalChaosResult(algo.getResult())
ancova = ot.ANCOVA(result, sample)
indices = ancova.getIndices()
uncorrelatedIndices = ancova.getUncorrelatedIndices()

for i in range(inputDimension):
    value = indices[i]
    print("ANCOVA index", i, "= %.6g" %
          value, "absolute error=%.6g" % fabs(value - Si[i][0]))
    value = uncorrelatedIndices[i]
    print("ANCOVA uncorrelated index", i, "= %.8f" %
          value, "absolute error=%.6g" % fabs(value - Si[i][1]))

# Compare full/sparse
ot.RandomGenerator.SetSeed(323)

model = ot.SymbolicFunction(['x1', 'x2', 'x3'], ['x1 + 0.56*x2 + 0.9*x3'])
distribution = ot.Normal(3)
distribution.setDescription(['x1', 'x2', 'x3'])

n_sample = 10
input_sample = distribution.getSample(n_sample)
output_sample = model(input_sample)

dim = 3
enumerateFunction = ot.LinearEnumerateFunction(dim)
polyCol = [0.]*dim
for i in range(dim):
    polyCol[i] = ot.StandardDistributionPolynomialFactory(
        distribution.getMarginal(i))

####### Chaos definition ######
multivariateBasis = ot.OrthogonalProductPolynomialFactory(
    polyCol, enumerateFunction)
indexMax = enumerateFunction.getStrataCumulatedCardinal(1)
strategy = ot.FixedStrategy(multivariateBasis, indexMax)

approximation_algorithm = ot.LeastSquaresMetaModelSelectionFactory(ot.LARS(),
                                                                   ot.CorrectedLeaveOneOut())
evaluationStrategy_sparse = ot.LeastSquaresStrategy(approximation_algorithm)
evaluationStrategy = ot.LeastSquaresStrategy()

# sparse and not sparse
chaos = ot.FunctionalChaosAlgorithm(input_sample, output_sample, distribution,
                                    strategy, evaluationStrategy)
chaos.run()
chaos_sparse = ot.FunctionalChaosAlgorithm(input_sample, output_sample, distribution,
                                           strategy, evaluationStrategy_sparse)
chaos_sparse.run()
print('indices/full=', chaos.getResult().getIndices())
print('indices/sparse=', chaos_sparse.getResult().getIndices())

ancova = ot.ANCOVA(chaos.getResult(), input_sample)
ancova_sparse = ot.ANCOVA(chaos_sparse.getResult(), input_sample)
print('Indice ancova, chaos normal : {:0.3f} {:0.3f} {:0.3f}'.format(
    *ancova.getIndices()))
print('Indice ancova, chaos sparse : {:0.3f} {:0.3f} {:0.3f}'.format(
    *ancova_sparse.getIndices()))



#! /usr/bin/env python

from __future__ import print_function, division
from openturns import *

TESTPREAMBLE()


dimension = 2
inVar = Description(dimension)
inVar[0] = "x1"
inVar[1] = "x2"
formula = Description(1)
formula[0] = "x1^3+1.5*x2^3-x1*x2"
model = SymbolicFunction(inVar, formula)
coll = []
formula[0] = "x1"
coll.append(SymbolicFunction(inVar, formula))
formula[0] = "x2"
coll.append(SymbolicFunction(inVar, formula))
formula[0] = "x1^2"
coll.append(SymbolicFunction(inVar, formula))
formula[0] = "x2^2"
coll.append(SymbolicFunction(inVar, formula))
basis = Basis(coll)
indices = list(range(len(coll)))
size = 5
inputSample = Sample(size * size, dimension)
weight = Point(inputSample.getSize(), 1)
for i in range(inputSample.getSize()):
    inputSample[i] = Point(
        (float(i % size) / size, (1.0 * (i // size)) / size))
    weight[i] = (i % size + 1) * ((i // size) + 1)
penalizationFactor = 0.25
# Uniform weight, no penalization
algo = PenalizedLeastSquaresAlgorithm(
    inputSample, model(inputSample), Point(inputSample.getSize(), 1.0), coll, indices)
print("Uniform weight, no penalization")
print("Coefficients=", (algo.getCoefficients()))
print("Residual=%g" % algo.getResidual())
print("RelativeError=%g" % algo.getRelativeError())
# Uniform weight, spherical penalization
algo = PenalizedLeastSquaresAlgorithm(
    inputSample, model(inputSample), Point(inputSample.getSize(), 1.0), coll, indices, penalizationFactor)
print("Uniform weight, spherical penalization")
print("Coefficients=", (algo.getCoefficients()))
print("Residual=%g" % algo.getResidual())
# Non uniform weight, spherical penalization
algo = PenalizedLeastSquaresAlgorithm(
    inputSample, model(inputSample), weight, coll, indices)
print("Non uniform weight, no penalization")
print("Coefficients=", (algo.getCoefficients()))
print("Residual=%g" % algo.getResidual())
algo = PenalizedLeastSquaresAlgorithm(
    inputSample, model(inputSample), weight, coll, indices, penalizationFactor)
print("Non uniform weight, spherical penalization")
print("Coefficients=", (algo.getCoefficients()))
print("Residual=%g" % algo.getResidual())
penalizationMatrix = CovarianceMatrix(4)
for i in range(4):
    penalizationMatrix[i, i] = 1.0
for i in range(3):
    penalizationMatrix[i, i + 1] = 1.0 / 8.0
algo = PenalizedLeastSquaresAlgorithm(
    inputSample, model(inputSample), weight, coll, indices, penalizationFactor, penalizationMatrix)
print("Non uniform weight, non spherical penalization")
print("Coefficients=", (algo.getCoefficients()))
print("Residual=%g" % algo.getResidual())



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # Instanciate one distribution object
    distribution = NormalGamma(1.0, 2.0, 3.0, 4.0)
    print("Distribution ", repr(distribution))
    print("Distribution ", distribution)

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", oneRealization)

    # Test for sampling
    size = 10000
    oneSample = distribution.getSample(size)
    print("oneSample first=", oneSample[0], " last=", oneSample[size - 1])
    print("mean=", oneSample.computeMean())
    print("covariance=", oneSample.computeCovariance())

    # Define a point
    point = [1.5] * distribution.getDimension()
    print("Point= ", point)

    # Show PDF and CDF of point
    eps = 1e-5
    DDF = distribution.computeDDF(point)
    print("ddf     =", DDF)
    LPDF = distribution.computeLogPDF(point)
    print("log pdf=%.6f" % LPDF)
    PDF = distribution.computePDF(point)
    print("pdf=%.6f" % PDF)
    CDF = distribution.computeCDF(point)
    print("cdf=%.6f" % CDF)
    CCDF = distribution.computeComplementaryCDF(point)
    print("ccdf=%.6f" % CCDF)
    Survival = distribution.computeSurvivalFunction(point)
    print("survival=%.6f" % Survival)
    InverseSurvival = distribution.computeInverseSurvivalFunction(0.95)
    print("Inverse survival=", InverseSurvival)
    print("Survival(inverse survival)=%.6f" %
          distribution.computeSurvivalFunction(InverseSurvival))
    PDFgr = distribution.computePDFGradient(point)
    print("pdf gradient     =", PDFgr)
    PDFgrFD = Point(4)
    PDFgrFD[0] = (NormalGamma(distribution.getMu() + eps, distribution.getKappa(), distribution.getAlpha(), distribution.getBeta()).computePDF(point)
                  - NormalGamma(distribution.getMu() - eps, distribution.getKappa(), distribution.getAlpha(), distribution.getBeta()).computePDF(point)) / (2.0 * eps)
    PDFgrFD[1] = (NormalGamma(distribution.getMu(), distribution.getKappa() + eps, distribution.getAlpha(), distribution.getBeta()).computePDF(point)
                  - NormalGamma(distribution.getMu(), distribution.getKappa() - eps, distribution.getAlpha(), distribution.getBeta()).computePDF(point)) / (2.0 * eps)
    PDFgrFD[2] = (NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha() + eps, distribution.getBeta()).computePDF(point)
                  - NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha() - eps, distribution.getBeta()).computePDF(point)) / (2.0 * eps)
    PDFgrFD[3] = (NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha(), distribution.getBeta() + eps).computePDF(point)
                  - NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha(), distribution.getBeta() - eps).computePDF(point)) / (2.0 * eps)
    print("pdf gradient (FD)=", PDFgrFD)
    CDFgr = distribution.computeCDFGradient(point)
    print("cdf gradient     =", CDFgr)
    CDFgrFD = Point(4)
    CDFgrFD[0] = (NormalGamma(distribution.getMu() + eps, distribution.getKappa(), distribution.getAlpha(), distribution.getBeta()).computeCDF(point)
                  - NormalGamma(distribution.getMu() - eps, distribution.getKappa(), distribution.getAlpha(), distribution.getBeta()).computeCDF(point)) / (2.0 * eps)
    CDFgrFD[1] = (NormalGamma(distribution.getMu(), distribution.getKappa() + eps, distribution.getAlpha(), distribution.getBeta()).computeCDF(point)
                  - NormalGamma(distribution.getMu(), distribution.getKappa() - eps, distribution.getAlpha(), distribution.getBeta()).computeCDF(point)) / (2.0 * eps)
    CDFgrFD[2] = (NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha() + eps, distribution.getBeta()).computeCDF(point)
                  - NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha() - eps, distribution.getBeta()).computeCDF(point)) / (2.0 * eps)
    CDFgrFD[3] = (NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha(), distribution.getBeta() + eps).computeCDF(point)
                  - NormalGamma(distribution.getMu(), distribution.getKappa(), distribution.getAlpha(), distribution.getBeta() - eps).computeCDF(point)) / (2.0 * eps)
    print("cdf gradient (FD)=", CDFgrFD)
    quantile = distribution.computeQuantile(0.95)
    print("quantile=", quantile)
    print("cdf(quantile)=%.6f" % distribution.computeCDF(quantile))
    print("entropy=%.6f" % distribution.computeEntropy())
    # Confidence regions
    probability, threshold = distribution.computeMinimumVolumeIntervalWithMarginalProbability(
        0.95)
    print("Minimum volume interval=", probability)
    print("threshold=%.6f" % threshold)
    levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(
        0.95)
    print("Minimum volume level set=", levelSet)
    print("beta=%.6f" % beta)
    interval, beta = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(
        0.95)
    print("Bilateral confidence interval=", interval)
    print("beta=%.6f" % beta)
    interval, beta = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, False)
    print("Unilateral confidence interval (lower tail)=", interval)
    print("beta=%.6f" % beta)
    interval, beta = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, True)
    print("Unilateral confidence interval (upper tail)=", interval)
    print("beta=%.6f" % beta)
    mean = distribution.getMean()
    print("mean=", mean)
    standardDeviation = distribution.getStandardDeviation()
    print("standard deviation=", standardDeviation)
    skewness = distribution.getSkewness()
    print("skewness=", skewness)
    kurtosis = distribution.getKurtosis()
    print("kurtosis=", kurtosis)
    covariance = distribution.getCovariance()
    print("covariance=", covariance)
    correlation = distribution.getCorrelation()
    print("correlation=", correlation)
    # These computations take too much time for a test
    # CovarianceMatrix spearman = distribution.getSpearmanCorrelation()
    # print("spearman=", spearman)
    # CovarianceMatrix kendall = distribution.getKendallTau()
    # print("kendall=", kendall)

    parameters = distribution.getParametersCollection()
    print("parameters=", parameters)
    for i in range(6):
        print("standard moment n=", i, ", value=",
              distribution.getStandardMoment(i))
    print("Standard representative=", distribution.getStandardRepresentative())

except:
    import sys
    print("t_NormalGamma_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot


# Default dimension parameter to evaluate the model
inputDimension = 1
outputDimension = 1

# Amplitude values
amplitude = [1.0] * outputDimension
# Scale values
scale = [1.0] * inputDimension

tmin = 0.0
step = 0.1
n = 11

myTimeGrid = ot.RegularGrid(tmin, step, n)
size = 25

# Second order model with parameters
myCovModel = ot.ExponentialModel(scale, amplitude)
print("myCovModel = ", myCovModel)

myProcess1 = ot.GaussianProcess(myCovModel, myTimeGrid)
print("myProcess1 = ", myProcess1)
print("is stationary? ", myProcess1.isStationary())
myProcess1.setSamplingMethod(0)
print("mean over ", size, " realizations = ",
      myProcess1.getSample(size).computeMean())
myProcess1.setSamplingMethod(2)
print("mean over ", size, " realizations = ",
      myProcess1.getSample(size).computeMean())

# With constant trend
trend = ot.TrendTransform(ot.SymbolicFunction("t", "4.0"), myTimeGrid)
myProcess2 = ot.GaussianProcess(trend, myCovModel, myTimeGrid)
myProcess2.setSamplingMethod(2)
print("myProcess2 = ", myProcess2)
print("is stationary? ", myProcess2.isStationary())
print("mean over ", size, " realizations= ",
      myProcess2.getSample(size).computeMean())

# With varying trend
trend3 = ot.TrendTransform(ot.SymbolicFunction("t", "sin(t)"), myTimeGrid)
myProcess3 = ot.GaussianProcess(trend3, myCovModel, myTimeGrid)
print("myProcess3 = ", myProcess3)
print("is stationary? ", myProcess3.isStationary())
myProcess3.setSamplingMethod(0)
print("mean over ", size, " realizations = ",
      myProcess3.getSample(size).computeMean())
myProcess3.setSamplingMethod(2)
print("mean over ", size, " realizations = ",
      myProcess3.getSample(size).computeMean())

cov = ot.ExponentialModel([2.0] * inputDimension, [1.0, 2.0, 3.0])
model = ot.GaussianProcess(cov, myTimeGrid)
print('model=', model)
print('marginal=', model.getMarginal([0, 2]))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    # Construction based on two functions
    # Create an instance */
    myFunc = PointToPointConnection(
        SymbolicFunction("x", "sin(x)"), SymbolicFunction("x", "cos(x)"))

    print("myFunc=", myFunc)
    # Get the input and output description */
    print("myFunc input description=", myFunc.getInputDescription())
    print("myFunc output description=", myFunc.getOutputDescription())
    # Get the input and output dimension */
    print("myFunc input dimension=", myFunc.getInputDimension())
    print("myFunc output dimension=", myFunc.getOutputDimension())
    # Connection on a point */
    point = [1.0] * myFunc.getInputDimension()
    print("point=", point)
    print("myFunc(point)=", myFunc(point))
    print("myFunc.gradient(point)=", myFunc.gradient(point))
    print("myFunc.hessian(point)=", myFunc.hessian(point))
    # Get the number of calls */
    print("called ", myFunc.getCallsNumber(), " times")

    # Construction based on a PointToFieldFunction followed by a FieldToPointFunction
    # Create a KarhunenLoeveResult
    mesh = IntervalMesher([9]).build(Interval(-1.0, 1.0))
    cov1D = AbsoluteExponential([1.0])
    algo = KarhunenLoeveP1Algorithm(mesh, cov1D, 1.0e-2)
    algo.run()
    result = algo.getResult()
    # Create a PointToFieldFunction
    lifting = KarhunenLoeveLifting(result)
    # Create a FieldToPointFunction
    projection = KarhunenLoeveProjection(result)
    # Create an instance */
    myFunc = PointToPointConnection(projection, lifting)

    print("myFunc=", myFunc)
    # Get the input and output description */
    print("myFunc input description=", myFunc.getInputDescription())
    print("myFunc output description=", myFunc.getOutputDescription())
    # Get the input and output dimension */
    print("myFunc input dimension=", myFunc.getInputDimension())
    print("myFunc output dimension=", myFunc.getOutputDimension())
    # Connection on a point */
    point = [1.0] * myFunc.getInputDimension()
    print("point=", point)
    print("myFunc(point)=", myFunc(point))
    print("myFunc.gradient(point)=", myFunc.gradient(point).clean(1.0e-8))
    print("myFunc.hessian(point)=", myFunc.hessian(point).clean(1.0e-7))
    # Get the number of calls */
    print("called ", myFunc.getCallsNumber(), " times")

except:
    import sys
    print("t_PointToPointConnection_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ref = ot.SymbolicFunction("x", "sin(x)")
size = 12
locations = [0.0] * size
values = [0.0] * size
derivatives = [0.0] * size
# Build locations/values/derivatives with non-increasing locations
for i in range(size):
    locations[i] = 10.0 * i * i / (size - 1.0) / (size - 1.0)
    values[i] = ref([locations[i]])[0]
    derivatives[i] = ref.gradient([locations[i]])[0, 0]

evaluation = ot.PiecewiseHermiteEvaluation(
    locations, values, derivatives)
print("evaluation=", evaluation)
# Check the values
X = [[-1.0 + 12.0 * i / (2.0 * size - 1.0)] for i in range(2 * size)]
for x in X:
    print("f( %.12g )=" % x[0], evaluation(x), ", ref=", ref(x))
Y = evaluation(X)
print(Y)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

size = 100
dim = 10
R = ot.CorrelationMatrix(dim)
for i in range(dim):
    for j in range(i):
        R[i, j] = (i + j + 1.0) / (2.0 * dim)

mean = [2.0] * dim
sigma = [3.0] * dim
distribution = ot.Normal(mean, sigma, R)

sample = distribution.getSample(size)
sampleX = sample.getMarginal(range(1, dim))
sampleY = sample.getMarginal(0)

sampleZ = ot.Sample(size, 1)
for i in range(size):
    sampleZ[i, 0] = sampleY[i, 0]**2
print("LinearModelFisher pvalue=%1.2g" %
      ot.LinearModelTest.LinearModelFisher(sampleY, sampleZ).getPValue())
print("LinearModelResidualMean pvalue=%1.2g" %
      ot.LinearModelTest.LinearModelResidualMean(sampleY, sampleZ).getPValue())

# Durbin Watson
ot.RandomGenerator.SetSeed(5415)
eps = ot.Normal(0, 20)
f = ot.SymbolicFunction('x', '5+2*x+x^2-0.1*x^3')
N = 15
x = ot.Sample([[0], [1.42857], [2.85714], [4.28571], [5.71429], [7.14286],
               [8.57143], [10], [11.4286], [12.8571], [14.2857], [15.7143],
               [17.1429], [18.5714], [20]])
y = f(x) + eps.getSample(N)
linmodel = ot.LinearModelAlgorithm(x, y).getResult().getCoefficients()
dwTest = ot.LinearModelTest.LinearModelDurbinWatson(x, y)
print('Durbin Watson = ', dwTest)


selection = ot.Indices(5)
selection.fill()

selection2 = ot.Indices(1, 0)
sampleX0 = sampleX.getMarginal(0)

# Regression test between 2 samples : firstSample of dimension n and
# secondSample of dimension 1. If firstSample[i] is the numerical sample
# extracted from firstSample (ith coordinate of each point of the
# numerical sample), PartialRegression performs the Regression test
# simultaneously on all firstSample[i] and secondSample, for i in the
# selection. The Regression test tests ifthe regression model between two
# scalar numerical samples is significant. It is based on the deviation
# analysis of the regression. The t-test is used.

# The two tests must be equal
print('PartialRegressionX0Y=', ot.LinearModelTest.PartialRegression(
    sampleX, sampleY, selection2, 0.10))
print('FullRegressionX0Y=', ot.LinearModelTest.FullRegression(
    sampleX0, sampleY, 0.10))

print('PartialRegressionXY=', ot.LinearModelTest.PartialRegression(
    sampleX, sampleY, selection, 0.10))

# Regression test between 2 samples : firstSample of dimension n and
# secondSample of dimension 1. If firstSample[i] is the numerical sample
# extracted from firstSample (ith coordinate of each point of the
# numerical sample), FullRegression performs the Regression test
# simultaneously on all firstSample[i] and secondSample. The Regression
# test tests if the regression model between two scalar numerical samples
# is significant. It is based on the deviation analysis of the regression.
# The t-test is used.

print('FullRegressionXZ=', ot.LinearModelTest.FullRegression(
    sampleX, sampleY, 0.10))
# print('FullRegressionZZ=', ot.LinearModelTest.FullRegression(
# sampleZ, sampleZ, 0.10))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)
PlatformInfo.SetNumericalPrecision(3)

try:
    distributions = [Student(3.5, 2.5)]
    R = CorrelationMatrix([[1.0, 0.5], [0.5, 1.0]])
    distributions.append(Student(4.5, [2.5]*2, [1.5]*2, R))
    R = CorrelationMatrix(10)
    mu = Point(10)
    sigma = Point(10)
    for i in range(10):
        mu[i] = 0.5 * i
        sigma[i] = 0.5 + i
        for j in range(i):
            R[i, j] = 1.0 / (i + j + 2.0)**2.0
    distributions.append(Student(4.5, mu, sigma, R))
    size = 10000
    factory = StudentFactory()
    for distribution in distributions:
        sample = distribution.getSample(size)
        estimatedDistribution = factory.build(sample)
        print("distribution=", distribution)
        print("Estimated distribution=", estimatedDistribution)
    estimatedDistribution = factory.build()
    print("Default distribution=", estimatedDistribution)
    estimatedStudent = factory.buildAsStudent(sample)
    print("Student          =", distribution)
    print("Estimated student=", estimatedStudent)
    estimatedStudent = factory.buildAsStudent()
    print("Default student=", estimatedStudent)

except:
    import sys
    print("t_StudentFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    distribution = Laplace(-1.3, 2.5)
    size = 10000
    sample = distribution.getSample(size)
    factory = LaplaceFactory()
    estimatedDistribution = factory.build(sample)
    print("distribution=", repr(distribution))
    print("Estimated distribution=", repr(estimatedDistribution))
    estimatedDistribution = factory.build()
    print("Default distribution=", estimatedDistribution)
    estimatedDistribution = factory.build(
        distribution.getParameter())
    print("Distribution from parameters=", estimatedDistribution)
    estimatedLaplace = factory.buildAsLaplace(sample)
    print("Laplace          =", distribution)
    print("Estimated laplace=", estimatedLaplace)
    estimatedLaplace = factory.buildAsLaplace()
    print("Default laplace=", estimatedLaplace)
    estimatedLaplace = factory.buildAsLaplace(
        distribution.getParameter())
    print("Laplace from parameters=", estimatedLaplace)

except:
    import sys
    print("t_LaplaceFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot


dim = 2

# We create an empty series
ts1 = ot.TimeSeries(0, dim)
ts1.setName('Ts1')

# We populate the empty ts
for p in range(3):
    pt = ot.Point(dim)
    for i in range(dim):
        pt[i] = 10. * (p + 1) + i
    ts1.add(pt)

print('ts1=', ts1)
print('len=', len(ts1))

# We get the second element of the ts
secondElement = ts1[1]
print('second element=', secondElement)

# We set the third element to a valid new element
newPoint = ot.Point(dim + 1)
for i in range(dim):
    newPoint[i + 1] = 1000. * (i + 1)
ts1[2] = newPoint
print('ts1=', ts1)

# We get the values of the second element of the ts
values1 = ts1.getValueAtIndex(1)
print('values of the second element=', values1)

# We set new values for the second element of the ts
newValues = values1 * 2.
ts1.setValueAtIndex(1, newValues)
values2 = ts1.getValueAtIndex(1)
print('ts1=', ts1)
print('modified values of the second element=',
      values2, ' should be ', newValues)

# We get the values of the second element of the ts
tree = ot.KDTree(ts1.getMesh().getVertices())
values3 = ts1.getValueAtIndex(tree.query([-1.0]))
print('values at t=-1.0 =', values3)
values4 = ts1.getValueAtIndex(tree.query([1.45]))
print('values at t=1.45 =', values4)
values5 = ts1.getValueAtIndex(tree.query([1.54]))
print('values at t=1.54 =', values5)
values6 = ts1.getValueAtIndex(tree.query([14.5]))
print('values at t=14.5 =', values6)

# We set new values for the third element of the ts
ts1.setValueAtIndex(tree.query([1.54]), values6 * -1.0)
print('ts1=', ts1)

ts2 = ot.TimeSeries(10, dim)
print('ts2=', ts2)

# ts2[5] = point2
# print 'ts2=', ts2

try:
    # We get the tenth element of the ts
    # THIS SHOULD NORMALLY FAIL
    tenthElement = ts1.at(9)
except:
    print('Expected failure')

tg1 = ot.RegularGrid(0.0, 0.1, 11)
ts3 = ot.TimeSeries(tg1, dim)
print('ts3=', ts3)

tg2 = ot.RegularGrid(0.0, 0.2, 6)
ts4 = ot.TimeSeries(tg2, dim)
print('ts4=', ts4)

# We append a sample to a time series
ts5 = ot.TimeSeries(3, dim)
ns1 = ot.Sample(3, [99.9] * dim)
print('ts5=', ts5)
ts5.add(ns1)
print('ts5=', ts5)

# We retrieve the values of the time series as a sample
ns2 = ts5.getValues()
print('ns2=', ns2)

# We build a time series from a time grid and a sample
ns3 = ts4.getValues()
ts6 = ot.TimeSeries(tg2, ns3)
print('ts6=', ts6)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    ResourceMap.SetAsUnsignedInteger(
        "MeixnerDistribution-CDFIntegrationNodesNumber", 8)
    ResourceMap.SetAsUnsignedInteger(
        "MeixnerDistribution-CDFDiscretization", 100)
    ResourceMap.SetAsScalar(
        "MeixnerDistribution-MaximumAbsoluteError", 1.0e-6)
    ResourceMap.SetAsScalar(
        "MeixnerDistribution-MaximumRelativeError", 1.0e-6)
    ResourceMap.SetAsScalar(
        "MeixnerDistribution-MaximumConstraintError", 1.0e-6)
    ResourceMap.SetAsScalar(
        "MeixnerDistribution-MaximumObjectiveError", 1.0e-6)
    distribution = MeixnerDistribution(1.5, 0.5, 2.5, -0.5)
    size = 1000
    sample = distribution.getSample(size)
    factory = MeixnerDistributionFactory()
    covariance = CovarianceMatrix()
    # Distribution estimatedDistribution(factory.build(sample, covariance))
    estimatedDistribution = factory.build(sample)
    print("Distribution          =", distribution)
    print("Estimated distribution=", estimatedDistribution)
    # print "Covariance=", covariance
    estimatedDistribution = factory.build()
    print("Default distribution=", estimatedDistribution)
    estimatedDistribution = factory.build(
        distribution.getParameter())
    print("Distribution from parameters=", estimatedDistribution)
    estimatedMeixnerDistribution = factory.buildAsMeixnerDistribution(sample)
    print("MeixnerDistribution          =", distribution)
    print("Estimated meixnerDistribution=", estimatedMeixnerDistribution)
    estimatedMeixnerDistribution = factory.buildAsMeixnerDistribution()
    print("Default meixnerDistribution=", estimatedMeixnerDistribution)
    estimatedMeixnerDistribution = factory.buildAsMeixnerDistribution(
        distribution.getParameter())
    print("MeixnerDistribution from parameters=", estimatedMeixnerDistribution)

except:
    import sys
    print("t_MeixnerDistributionFactory_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()


# Instanciate one distribution object
distribution = ot.Pareto(7.5, 5.0, -7.0)

print("Distribution ", repr(distribution))
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", repr(oneRealization))

# Test for sampling
size = 10000
oneSample = distribution.getSample(size)
print("oneSample first=", repr(
    oneSample[0]), " last=", repr(oneSample[size - 1]))
print("mean=", repr(oneSample.computeMean()))
print("covariance=", repr(oneSample.computeCovariance()))
print("skewness=", repr(oneSample.computeSkewness()))
print("kurtosis=", repr(oneSample.computeKurtosis()))

size = 100
for i in range(2):
    msg = ''
    if ot.FittingTest.Kolmogorov(distribution.getSample(size), distribution).getBinaryQualityMeasure():
        msg = "accepted"
    else:
        msg = "rejected"
    print("Kolmogorov test for the generator, sample size=", size, " is", msg)
    size *= 10

# Define a point
point = ot.Point(distribution.getDimension(), 1.0)
print("Point= ", repr(point))
print("range= ", distribution.getRange())

# Show PDF and CDF of point
eps = 1e-5
DDF = distribution.computeDDF(point)
print("ddf     =", repr(DDF))
print("ddf (FD)=", repr(ot.Point(1, (distribution.computePDF(
    point + ot.Point(1, eps)) - distribution.computePDF(point + ot.Point(1, -eps))) / (2.0 * eps))))
PDF = distribution.computePDF(point)
print("pdf     = %.12g" % PDF)
print("pdf (FD)= %.9f" % ((distribution.computeCDF(point + ot.Point(1, eps)) -
                           distribution.computeCDF(point + ot.Point(1, -eps))) / (2.0 * eps), ))

CDF = distribution.computeCDF(point)
print("cdf= %.12g" % CDF)
# CF = distribution.computeCharacteristicFunction( point[0] )
# print "characteristic function=", CF
PDFgr = distribution.computePDFGradient(point)
print("pdf gradient     =", repr(PDFgr))
PDFgrFD = ot.Point(3)
PDFgrFD[0] = (ot.Pareto(distribution.getBeta() + eps, distribution.getAlpha(), distribution.getGamma()).computePDF(point) -
              ot.Pareto(distribution.getBeta() - eps, distribution.getAlpha(), distribution.getGamma()).computePDF(point)) / (2.0 * eps)
PDFgrFD[1] = (ot.Pareto(distribution.getBeta(), distribution.getAlpha() + eps, distribution.getGamma()).computePDF(point) -
              ot.Pareto(distribution.getBeta(), distribution.getAlpha() - eps, distribution.getGamma()).computePDF(point)) / (2.0 * eps)
PDFgrFD[2] = (ot.Pareto(distribution.getBeta(), distribution.getAlpha(), distribution.getGamma() + eps).computePDF(point) -
              ot.Pareto(distribution.getBeta(), distribution.getAlpha(), distribution.getGamma() - eps).computePDF(point)) / (2.0 * eps)
print("pdf gradient (FD)=", repr(PDFgrFD))
CDFgr = distribution.computeCDFGradient(point)
print("cdf gradient     =", repr(CDFgr))
CDFgrFD = ot.Point(3)
CDFgrFD[0] = (ot.Pareto(distribution.getBeta() + eps, distribution.getAlpha(), distribution.getGamma()).computeCDF(point) -
              ot.Pareto(distribution.getBeta() - eps, distribution.getAlpha(), distribution.getGamma()).computeCDF(point)) / (2.0 * eps)
CDFgrFD[1] = (ot.Pareto(distribution.getBeta(), distribution.getAlpha() + eps, distribution.getGamma()).computeCDF(point) -
              ot.Pareto(distribution.getBeta(), distribution.getAlpha() - eps, distribution.getGamma()).computeCDF(point)) / (2.0 * eps)
CDFgrFD[2] = (ot.Pareto(distribution.getBeta(), distribution.getAlpha(), distribution.getGamma() + eps).computeCDF(point) -
              ot.Pareto(distribution.getBeta(), distribution.getAlpha(), distribution.getGamma() - eps).computeCDF(point)) / (2.0 * eps)
print("cdf gradient (FD)=", repr(CDFgrFD))
quantile = distribution.computeQuantile(0.95)
print("quantile=", repr(quantile))
print("cdf(quantile)=", distribution.computeCDF(quantile))
# Get 95% survival function
inverseSurvival = ot.Point(
    distribution.computeInverseSurvivalFunction(0.95))
print("InverseSurvival=", repr(inverseSurvival))
print("Survival(inverseSurvival)=%.6f" %
      distribution.computeSurvivalFunction(inverseSurvival))
print("entropy=%.6f" % distribution.computeEntropy())

# Confidence regions
interval, threshold = distribution.computeMinimumVolumeIntervalWithMarginalProbability(
    0.95)
print("Minimum volume interval=", interval)
print("threshold=", ot.Point(1, threshold))
levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(
    0.95)
print("Minimum volume level set=", levelSet)
print("beta=", ot.Point(1, beta))
interval, beta = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(
    0.95)
print("Bilateral confidence interval=", interval)
print("beta=", ot.Point(1, beta))
interval, beta = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
    0.95, False)
print("Unilateral confidence interval (lower tail)=", interval)
print("beta=", ot.Point(1, beta))
interval, beta = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
    0.95, True)
print("Unilateral confidence interval (upper tail)=", interval)
print("beta=", ot.Point(1, beta))

mean = distribution.getMean()
print("mean=", repr(mean))
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", repr(standardDeviation))
skewness = distribution.getSkewness()
print("skewness=", repr(skewness))
kurtosis = distribution.getKurtosis()
print("kurtosis=", repr(kurtosis))
covariance = distribution.getCovariance()
print("covariance=", repr(covariance))
parameters = distribution.getParametersCollection()
print("parameters=", repr(parameters))
for i in range(6):
    try:
        value = distribution.getStandardMoment(i)
        print("standard moment n=", i, " value=", value)
    except RuntimeError as ex:
        print(ex)
print("Standard representative=",
      distribution.getStandardRepresentative())



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import openturns.testing as ott
ot.TESTPREAMBLE()


# Instanciate one distribution object
distribution = ot.DiscreteCompoundDistribution(
    ot.Bernoulli(0.5), ot.Poisson(20.0))
upper_bound = int(distribution.getRange().getUpperBound()[0])
print("Upper bound : {!r}".format(upper_bound))

# Compare with mathematically equal distribution
poisson_distribution = ot.Poisson(10.0)

for i in range(upper_bound):
    ott.assert_almost_equal(distribution.computePDF(
        [i]), poisson_distribution.computePDF([i]))

print("Distribution ", repr(distribution))
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", repr(oneRealization))

# Test for sampling
size = 10000
oneSample = distribution.getSample(size)
print("oneSample first=", repr(
    oneSample[0]), " last=", repr(oneSample[size - 1]))
print("mean=", repr(oneSample.computeMean()))
print("covariance=", repr(oneSample.computeCovariance()))

# quantile
quantile = distribution.computeQuantile(0.95)
print("quantile=", repr(quantile))
print("cdf(quantile)=%.6f" % distribution.computeCDF(quantile))
print("entropy=%.6f" % distribution.computeEntropy())
mean = distribution.getMean()
print("mean=", repr(mean))
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", repr(standardDeviation))
skewness = distribution.getSkewness()
print("skewness=", repr(skewness))
kurtosis = distribution.getKurtosis()
print("kurtosis=", repr(kurtosis))
covariance = distribution.getCovariance()
print("covariance=", repr(covariance))
for i in range(6):
    print("standard moment n=", i, " value=",
          distribution.getStandardMoment(i))
print("Standard representative=", distribution.getStandardRepresentative())
#print("probabilities=", distribution.getProbabilities())



#!/usr/bin/env python

from __future__ import print_function
import openturns as ot

for lx in range(10):
    x = 2**lx
    print('x=', x, 'log2=', ot.SpecFunc.Log2(x))

for lx in range(10):
    x = 2**lx - 1
    print('x=', x, 'bits=', ot.SpecFunc.BitCount(x))

for x in [0, 1, 2, 3, 4, 10, 100, 1000]:
    print('x=', x, 'nextpow2=', ot.SpecFunc.NextPowerOfTwo(x))

for n in [4, 5, 17]:
    for k in [2, 0, 7, 3]:
        if k <= n:
            print('n=', n, 'k=', k, 'binom=',
                  ot.SpecFunc.BinomialCoefficient(n, k))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
import openturns as ot
import openturns.testing as ott


def printResults(result, problemName):
    print("*** {} completed:".format(problemName))
    print("      -- Optimal point = ", result.getOptimalPoint())
    print("      -- Optimal value = ", result.getOptimalValue())
    print("      -- Iteration number = ", result.getIterationNumber())
    print("      -- Evaluation number = ", result.getEvaluationNumber())
    print("      -- Absolute error = {:.6e}".format(result.getAbsoluteError()))
    print("      -- Relative error = {:.6e}".format(result.getRelativeError()))
    print("      -- Residual error = {:.6e}".format(result.getResidualError()))
    print(
        "      -- Constraint error = {:.6e}".format(result.getConstraintError()))


# Define the problems based on Rosebrock function
rosenbrock = ot.SymbolicFunction(['x1', 'x2'], ['(1-x1)^2+100*(x2-x1^2)^2'])

unboundedProblem = ot.OptimizationProblem(rosenbrock)

notConstrainingBounds = ot.Interval([-5.0, -5.0], [5.0, 5.0])
notConstrainingBoundsProblem = ot.OptimizationProblem(
    rosenbrock, ot.Function(), ot.Function(), notConstrainingBounds)

constrainingBounds = ot.Interval([0.0, -2.0], [5.0, 0.5])
constrainingBoundsProblem = ot.OptimizationProblem(
    rosenbrock, ot.Function(), ot.Function(), constrainingBounds)

start = [3.0, -1.5]
unboundedPref = [1.0, 1.0]
boundedPref = [0.70856, 0.5]

## CONJUGATE GRADIENT ALGORITHM ##

# Unbounded CG
unboundedCg = ot.Dlib(unboundedProblem, 'CG')
unboundedCg.setStartingPoint(start)
unboundedCg.setMaximumIterationNumber(10000)
unboundedCg.setMaximumEvaluationNumber(100000)
unboundedCg.run()
printResults(unboundedCg.getResult(), 'Unbounded CG')

# Non-contraining bounds CG
notConstrainingBoundsCg = ot.Dlib(notConstrainingBoundsProblem, "CG")
notConstrainingBoundsCg.setStartingPoint(start)
notConstrainingBoundsCg.setMaximumIterationNumber(10000)
notConstrainingBoundsCg.setMaximumEvaluationNumber(100000)
notConstrainingBoundsCg.run()
printResults(notConstrainingBoundsCg.getResult(), "Non-constraining bounds CG")

# Contraining bounds CG
constrainingBoundsCg = ot.Dlib(constrainingBoundsProblem, "CG")
constrainingBoundsCg.setStartingPoint(start)
constrainingBoundsCg.setMaximumIterationNumber(10000)
constrainingBoundsCg.setMaximumEvaluationNumber(100000)
constrainingBoundsCg.run()
printResults(constrainingBoundsCg.getResult(), "Constraining bounds CG")


## BFGS ALGORITHM ##

# Unbounded BFGS
unboundedBfgs = ot.Dlib(unboundedProblem, "BFGS")
unboundedBfgs.setStartingPoint(start)
unboundedBfgs.setMaximumIterationNumber(10000)
unboundedBfgs.setMaximumEvaluationNumber(100000)
unboundedBfgs.run()
printResults(unboundedBfgs.getResult(), "Unbounded BFGS")

# Non-contraining bounds BFGS
notConstrainingBoundsBfgs = ot.Dlib(notConstrainingBoundsProblem, "BFGS")
notConstrainingBoundsBfgs.setStartingPoint(start)
notConstrainingBoundsBfgs.setMaximumIterationNumber(10000)
notConstrainingBoundsBfgs.setMaximumEvaluationNumber(100000)
notConstrainingBoundsBfgs.run()
printResults(notConstrainingBoundsBfgs.getResult(),
             "Non-constraining bounds BFGS")

# Contraining bounds BFGS
constrainingBoundsBfgs = ot.Dlib(constrainingBoundsProblem, "BFGS")
constrainingBoundsBfgs.setStartingPoint(start)
constrainingBoundsBfgs.setMaximumIterationNumber(10000)
constrainingBoundsBfgs.setMaximumEvaluationNumber(100000)
constrainingBoundsBfgs.run()
printResults(constrainingBoundsBfgs.getResult(), "Constraining bounds BFGS")


## LBFGS ALGORITHM ##

# Unbounded LBFGS
unboundedLbfgs = ot.Dlib(unboundedProblem, "LBFGS")
unboundedLbfgs.setStartingPoint(start)
unboundedLbfgs.setMaximumIterationNumber(10000)
unboundedLbfgs.setMaximumEvaluationNumber(100000)
unboundedLbfgs.setMaxSize(15)
unboundedLbfgs.run()
printResults(unboundedLbfgs.getResult(), "Unbounded LBFGS")

# Non-contraining bounds LBFGS
notConstrainingBoundsLbfgs = ot.Dlib(notConstrainingBoundsProblem, "LBFGS")
notConstrainingBoundsLbfgs.setStartingPoint(start)
notConstrainingBoundsLbfgs.setMaximumIterationNumber(10000)
notConstrainingBoundsLbfgs.setMaximumEvaluationNumber(100000)
notConstrainingBoundsLbfgs.setMaxSize(15)
notConstrainingBoundsLbfgs.run()
printResults(notConstrainingBoundsLbfgs.getResult(),
             "Non-constraining bounds LBFGS")

# Contraining bounds LBFGS
constrainingBoundsLbfgs = ot.Dlib(constrainingBoundsProblem, "LBFGS")
constrainingBoundsLbfgs.setStartingPoint(start)
constrainingBoundsLbfgs.setMaximumIterationNumber(10000)
constrainingBoundsLbfgs.setMaximumEvaluationNumber(100000)
constrainingBoundsLbfgs.setMaxSize(15)
constrainingBoundsLbfgs.run()


## NEWTON ALGORITHM ##

# Unbounded BFGS
unboundedNewton = ot.Dlib(unboundedProblem, "Newton")
unboundedNewton.setStartingPoint(start)
unboundedNewton.setMaximumIterationNumber(10000)
unboundedNewton.setMaximumEvaluationNumber(100000)
unboundedNewton.run()
printResults(unboundedNewton.getResult(), "Unbounded Newton")

# Non-contraining bounds Newton
notConstrainingBoundsNewton = ot.Dlib(notConstrainingBoundsProblem, "Newton")
notConstrainingBoundsNewton.setStartingPoint(start)
notConstrainingBoundsNewton.setMaximumIterationNumber(10000)
notConstrainingBoundsNewton.setMaximumEvaluationNumber(100000)
notConstrainingBoundsNewton.run()
printResults(notConstrainingBoundsNewton.getResult(),
             "Non-constraining bounds Newton")

# Contraining bounds Newton
constrainingBoundsNewton = ot.Dlib(constrainingBoundsProblem, "Newton")
constrainingBoundsNewton.setStartingPoint(start)
constrainingBoundsNewton.setMaximumIterationNumber(10000)
constrainingBoundsNewton.setMaximumEvaluationNumber(100000)
constrainingBoundsNewton.run()
printResults(constrainingBoundsNewton.getResult(),
             "Constraining bounds Newton")


## TRUST REGION ALGORITHM ##
unboundedTrustRegion = ot.Dlib(unboundedProblem, "TrustRegion")
unboundedTrustRegion.setStartingPoint(start)
unboundedTrustRegion.setMaximumIterationNumber(10000)
unboundedTrustRegion.setMaximumEvaluationNumber(100000)
unboundedTrustRegion.run()
printResults(unboundedTrustRegion.getResult(), "Unbounded TrustRegion")


ott.assert_almost_equal(
    unboundedCg.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    notConstrainingBoundsCg.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    constrainingBoundsCg.getResult().getOptimalPoint(), boundedPref, 5e-2)
ott.assert_almost_equal(
    unboundedBfgs.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    notConstrainingBoundsBfgs.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    constrainingBoundsBfgs.getResult().getOptimalPoint(), boundedPref, 5e-2)
ott.assert_almost_equal(
    unboundedLbfgs.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    notConstrainingBoundsLbfgs.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    constrainingBoundsLbfgs.getResult().getOptimalPoint(), boundedPref, 5e-2)
ott.assert_almost_equal(
    unboundedNewton.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    notConstrainingBoundsNewton.getResult().getOptimalPoint(), unboundedPref, 5e-2)
ott.assert_almost_equal(
    constrainingBoundsNewton.getResult().getOptimalPoint(), boundedPref, 5e-2)
ott.assert_almost_equal(
    unboundedTrustRegion.getResult().getOptimalPoint(), unboundedPref, 5e-2)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:

    # Tmin , time step and step number for TimeGrid
    Tmin = 0.0
    deltaT = 1.0
    N = 11
    timeGrid = RegularGrid(Tmin, deltaT, N)

    # Fixing the distribution for the WhiteNoise
    dist = Distribution(Uniform())
    print("dist = ", dist)

    # Building a process from a White Noise
    process = Process(WhiteNoise(dist))

    # print the process
    process.setTimeGrid(timeGrid)
    print("process = ", process)

    # Get a realization of the process
    timeSerie = process.getRealization()
    print("timeSerie = ", timeSerie)


except:
    import sys
    print("t_WhiteNoise_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    size = 10
    for dimension in range(1, 4):
        f = LinearEnumerateFunction(dimension)
        print("First", size, " values for dimension", dimension)
        for index in range(size):
            multiIndex = f(index)
            print("index=", index, ", multi-index=", multiIndex,
                  ", linear index=", f.inverse(multiIndex))
except:
    import sys
    print("t_LinearEnumerateFunction_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # Instanciate one distribution object
    first = -1.5
    l = [1.0, 0.7, 1.2, 0.9]
    h = [0.5, 1.5, 3.5, 2.5]
    t = [first]
    f = list()
    for i in range(len(l)):
        t.append(t[i] + l[i])
        f.append(h[i] * l[i])
    distribution = Histogram(t, f)
    print("Distribution ", repr(distribution))
    print("Distribution ", distribution)
    distribution = Histogram(first, l, h)
    print("Distribution ", repr(distribution))
    print("Distribution ", distribution)

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", repr(oneRealization))

    # Test for sampling
    size = 10000
    oneSample = distribution.getSample(size)
    print("oneSample first=", repr(oneSample[0]), " last=", repr(oneSample[1]))
    print("mean=", repr(oneSample.computeMean()))
    print("covariance=", repr(oneSample.computeCovariance()))

    size = 100
    for i in range(2):
        msg = ''
        if FittingTest.Kolmogorov(distribution.getSample(size), distribution).getBinaryQualityMeasure():
            msg = "accepted"
        else:
            msg = "rejected"
        print(
            "Kolmogorov test for the generator, sample size=", size, " is", msg)
        size *= 10

    # Define a point
    point = Point(distribution.getDimension(), 1.0)
    print("Point= ", repr(point))

    # Show PDF and CDF at point
    eps = 1e-5
    # derivative of PDF with regards its arguments
    DDF = distribution.computeDDF(point)
    print("ddf     =", repr(DDF))
    # by the finite difference technique
    print("ddf (FD)=", repr(Point(1, (distribution.computePDF(
        point + Point(1, eps)) - distribution.computePDF(point + Point(1, -eps))) / (2.0 * eps))))

    # PDF value
    LPDF = distribution.computeLogPDF(point)
    print("log pdf=%.6f" % LPDF)
    PDF = distribution.computePDF(point)
    print("pdf     =%.6f" % PDF)
    # by the finite difference technique from CDF
    print("pdf (FD)=%.6f" % ((distribution.computeCDF(point + Point(1, eps)) -
                              distribution.computeCDF(point + Point(1, -eps))) / (2.0 * eps)))

    # derivative of the PDF with regards the parameters of the distribution
    CDF = distribution.computeCDF(point)
    print("cdf=%.6f" % CDF)
    CCDF = distribution.computeComplementaryCDF(point)
    print("ccdf=%.6f" % CCDF)

    # quantile
    quantile = distribution.computeQuantile(0.95)
    print("quantile=", repr(quantile))
    print("cdf(quantile)=%.6f" % distribution.computeCDF(quantile))
    # Get 95% survival function
    inverseSurvival = Point(distribution.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print("Survival(inverseSurvival)=%.6f" %
          distribution.computeSurvivalFunction(inverseSurvival))
    print("entropy=%.6f" % distribution.computeEntropy())

    # Confidence regions
    interval, threshold = distribution.computeMinimumVolumeIntervalWithMarginalProbability(
        0.95)
    print("Minimum volume interval=", interval)
    print("threshold=", Point(1, threshold))
    levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(
        0.95)
    print("Minimum volume level set=", levelSet)
    print("beta=", Point(1, beta))
    interval, beta = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(
        0.95)
    print("Bilateral confidence interval=", interval)
    print("beta=", Point(1, beta))
    interval, beta = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, False)
    print("Unilateral confidence interval (lower tail)=", interval)
    print("beta=", Point(1, beta))
    interval, beta = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, True)
    print("Unilateral confidence interval (upper tail)=", interval)
    print("beta=", Point(1, beta))

    mean = distribution.getMean()
    print("mean=", repr(mean))
    covariance = distribution.getCovariance()
    print("covariance=", repr(covariance))
    parameters = distribution.getParametersCollection()
    print("parameters=", repr(parameters))
    for i in range(6):
        print("standard moment n=", i, " value=",
              distribution.getStandardMoment(i))
    print("Standard representative=", distribution.getStandardRepresentative())

    testSize = 0
    for i in range(testSize):
        q = RandomGenerator().Generate()
        if (fabs(q - distribution.computeCDF(distribution.computeQuantile(q))) > eps):
            print("q=%.6f" % q,  " quantile=%.6f" % distribution.computeQuantile(q)[
                  0],  " CDF(quantile)=%.6f" % distribution.computeCDF(distribution.computeQuantile(q)))

except:
    import sys
    print("t_Histogram.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    distribution = Normal(4)
    distribution.setMean([5.0]*4)
    size = 16
    experiment = LowDiscrepancyExperiment(HaltonSequence(), distribution, size)
    print("experiment = ", experiment)
    # Test sampling with weights
    sample, weights = experiment.generateWithWeights()
    print("sample  = ", sample)
    print("weights = ", weights)
    # Test sampling with reinitialization each time the distribution is
    # set (default behaviour)
    # sample 2 != sample
    print("sample2 = ", experiment.generate())
    experiment.setDistribution(distribution)
    # sample 3 == sample
    print("sample3 = ", experiment.generate())

    # Test sampling without reinitialization excepted when distribution
    # dimension changes
    experiment = LowDiscrepancyExperiment(
        HaltonSequence(), distribution, size, False)
    print("sample  = ", experiment.generate())
    # sample 2 != sample
    print("sample2 = ", experiment.generate())
    experiment.setDistribution(distribution)
    # sample 3 != sample && sample 3 != sample 2
    print("sample3 = ", experiment.generate())
    # Test dimension change
    experiment.setDistribution(Normal())
    print("sample = ", experiment.generate())

    # Test constructor with no distribution and dimension>1
    experiment = LowDiscrepancyExperiment(HaltonSequence(2), size)
    print("sample = ", experiment.generate())
    # Test with dependent marginals
    R = CorrelationMatrix(4)
    for i in range(1, 4):
        R[i - 1, i] = 0.5
    distribution.setCorrelation(R)
    experiment = LowDiscrepancyExperiment(
        HaltonSequence(), distribution, size, False)
    print("sample = ", experiment.generate())
except:
    import sys
    print("t_LowDiscrepancyExperiment_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

stratas = 4
for dimension in range(1, 4):
    f = ot.NormInfEnumerateFunction(dimension)
    size = f.getStrataCumulatedCardinal(stratas)
    print("First", size, "values for dimension", dimension)
    for i in range(size):
        print('i=', i, 'f(i)=', f(i))



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
from math import sin

ot.TESTPREAMBLE()

try:

    # lm build
    print(
        "Fit y ~ 3 - 2 x + 0.05 * sin(x) model using 20 points (sin(x) ~ noise)")
    size = 20
    oneSample = ot.Sample(size, 1)
    twoSample = ot.Sample(size, 1)
    for i in range(size):
        oneSample[i, 0] = 7.0 * sin(-3.5 + (6.5 * i) / (size - 1.0)) + 2.0
        twoSample[i, 0] = -2.0 * oneSample[
            i, 0] + 3.0 + 0.05 * sin(oneSample[i, 0])

    test = ot.LinearModelAlgorithm(oneSample, twoSample)
    result = ot.LinearModelResult(test.getResult())
    print ("trend coefficients = ", result.getCoefficients())

    print("Fit y ~ 1 + 0.1 x + 10 x^2 model using 100 points")
    ot.RandomGenerator.SetSeed(0)
    size = 100
    # Define a linespace from 0 to 10 with size points
    # We use a Box expermient ==> remove 0 & 1 points
    experiment = ot.Box([size - 2])
    X = experiment.generate()
    # X is defined in [0,1]
    X *= [10]
    # Stack X2
    X2 = ot.Sample(X)
    for i in range(size):
        X2[i, 0] = X[i, 0] * X2[i, 0]
    X.stack(X2)
    # Define y
    Y = ot.Sample(size, 1)
    for i in range(size):
        Y[i, 0] = 1.0 + 0.1 * X[i, 0] + 10.0 * \
            X[i, 0] * X[i, 0] + 0.1 * ot.DistFunc.rNormal()
    test = ot.LinearModelAlgorithm(X, Y)
    result = test.getResult()
    print ("trend coefficients = ", result.getCoefficients())

except:
    import sys
    print("t_LinearModelAlgorithm_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:

    # Analytical construction
    analytical = SymbolicFunction(["x0", "x1"], ["x0+x1"])
    print("function=", repr(analytical))

    # Create indicator function
    indicator = IndicatorFunction(
        analytical, Less(), 0.0)
    # Does it work?
    x = Point(2, 1.0)
    value = analytical(x)[0]
    print("Value of the function=", repr(value),
          " value of the indicator=", repr(indicator(x)))
    x = Point(2, -1.0)
    value = analytical(x)[0]
    print("Value of the function=", repr(value),
          " value of the indicator=", repr(indicator(x)))

except:
    import sys
    print("t_Function_indicator.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

ot.PlatformInfo.SetNumericalPrecision(5)
size = 10000


for distribution in [ot.GeneralizedExtremeValue(2.0, 1.5, -0.15),
                     ot.GeneralizedExtremeValue(2.0, 1.5,  0.0)]:
                     # ot.GeneralizedExtremeValue(2.0, 1.5,  0.15)]:

    sample = distribution.getSample(size)
    factory = ot.GeneralizedExtremeValueFactory()
    estimatedDistribution = factory.build(sample)
    print("distribution=", repr(distribution))
    print("Estimated distribution=", repr(estimatedDistribution))

    estimatedDistribution = factory.build()
    print("Default distribution=", estimatedDistribution)
    estimatedDistribution = factory.build(distribution.getParameter())
    print("Distribution from parameters=", estimatedDistribution)
    estimatedGeneralizedExtremeValue = factory.buildAsGeneralizedExtremeValue(
        sample)
    print("GeneralizedExtremeValue          =", distribution)
    print("Estimated GeneralizedExtremeValue=",
          estimatedGeneralizedExtremeValue)
    estimatedGeneralizedExtremeValue = factory.buildAsGeneralizedExtremeValue()
    print("Default GeneralizedExtremeValue=", estimatedGeneralizedExtremeValue)
    estimatedGeneralizedExtremeValue = factory.buildAsGeneralizedExtremeValue(
        distribution.getParameter())
    print("GeneralizedExtremeValue from parameters=",
          estimatedGeneralizedExtremeValue)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()


class RVEC(ot.PythonRandomVector):

    def __init__(self):
        super(RVEC, self).__init__(2)
        self.setDescription(['R', 'S'])

    def getRealization(self):
        X = [ot.RandomGenerator.Generate(), 2 + ot.RandomGenerator.Generate()]
        return X

    def getSample(self, size):
        X = []
        for i in range(size):
            X.append(
                [ot.RandomGenerator.Generate(), 2 + ot.RandomGenerator.Generate()])
        return X

    def getMean(self):
        return [0.5, 2.5]

    def getCovariance(self):
        return [[0.0833333, 0.], [0., 0.0833333]]

    def isEvent(self):
        return False


R = RVEC()
print(R)


# Instance creation
myRV = ot.RandomVector(R)
print("myRV=", repr(myRV))

# Copy constructor
newRV = ot.RandomVector(myRV)

# Dimension
dim = myRV.getDimension()
print('dimension=', dim)

# Realization
X = myRV.getRealization()
print('realization=', X)

# Sample
X = myRV.getSample(5)
print('sample=', X)

# Mean
mean = myRV.getMean()
print('mean=', mean)

# Covariance
covariance = myRV.getCovariance()
print('covariance=', covariance)

isEvent = myRV.isEvent()
print('isEvent=', isEvent)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
from openturns.testing import assert_almost_equal

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)


def matrixToSample(matrix):
    '''Converts a matrix into a Sample, so that we can
    assert_almost_equal a Matrix'''
    size = matrix.getNbRows()
    dimension = matrix.getNbColumns()
    sample = ot.Sample(size, dimension)
    for i in range(size):
        for j in range(dimension):
            sample[i, j] = matrix[i, j]
    return sample


'''
Test the calibration of an exponential function with 3 parameters.
Test the three decompositions (QR, SVD, Cholesky) of the least
squares problem.
Test the local and global error covariances.
'''
m = 10
x = [[0.5 + i] for i in range(m)]

inVars = ["a", "b", "c", "x"]
formulas = ["a + b * exp(c * x)", "(a * x^2 + b) / (c + x^2)"]
model = ot.SymbolicFunction(inVars, formulas)
p_ref = [2.8, 1.2, 0.5]
params = [0, 1, 2]
modelX = ot.ParametricFunction(model, params, p_ref)
y = modelX(x)
y += ot.Normal([0.0]*2, [0.05]*2, ot.IdentityMatrix(2)).getSample(m)
candidate = [1.0]*3
priorCovariance = ot.CovarianceMatrix(3)
for i in range(3):
    priorCovariance[i, i] = 3.0 + (1.0 + i) * (1.0 + i)
    for j in range(i):
        priorCovariance[i, j] = 1.0 / (1.0 + i + j)
errorCovariance = ot.CovarianceMatrix(2)
for i in range(2):
    errorCovariance[i, i] = 2.0 + (1.0 + i) * (1.0 + i)
    for j in range(i):
        errorCovariance[i, j] = 1.0 / (1.0 + i + j)
globalErrorCovariance = ot.CovarianceMatrix(2 * m)
for i in range(2 * m):
    globalErrorCovariance[i, i] = 2.0 + (1.0 + i) * (1.0 + i)
    for j in range(i):
        globalErrorCovariance[i, j] = 1.0 / (1.0 + i + j)

methods = ["SVD", "QR", "Cholesky"]
for method in methods:
    print("method=", method)
    # 1. Check with local error covariance
    print("Local error covariance")
    algo = ot.GaussianLinearCalibration(
        modelX, x, y, candidate, priorCovariance, errorCovariance, method)
    algo.run()
    calibrationResult = algo.getResult()

    # Analysis of the results
    # Maximum A Posteriori estimator
    thetaMAP = calibrationResult.getParameterMAP()
    exactTheta = ot.Point([5.69186, 0.0832132, 0.992301])
    rtol = 1.e-2
    assert_almost_equal(thetaMAP, exactTheta, rtol)

    # Covariance matrix of theta
    thetaPosterior = calibrationResult.getParameterPosterior()
    covarianceThetaStar = matrixToSample(thetaPosterior.getCovariance())
    exactCovarianceTheta = ot.Sample(
        [[0.308302, -0.000665387, 6.81135e-05],
         [-0.000665387, 8.36243e-06, -8.86775e-07],
         [6.81135e-05, -8.86775e-07, 9.42234e-08]])
    assert_almost_equal(covarianceThetaStar, exactCovarianceTheta)

    # Check other fields
    print("result=", calibrationResult)

    # 2. Check with global error covariance
    print("Global error covariance")
    algo = ot.GaussianLinearCalibration(
        modelX, x, y, candidate, priorCovariance, globalErrorCovariance, method)
    algo.run()
    calibrationResult = algo.getResult()

    # Analysis of the results
    # Maximum A Posteriori estimator
    thetaMAP = calibrationResult.getParameterMAP()
    exactTheta = ot.Point([3.4397, 0.095908, 0.99096])
    rtol = 1.e-2
    assert_almost_equal(thetaMAP, exactTheta, rtol)

    # Covariance matrix of theta
    thetaPosterior = calibrationResult.getParameterPosterior()
    covarianceThetaStar = matrixToSample(thetaPosterior.getCovariance())
    exactCovarianceTheta = ot.Sample(
        [[1.27112112e+00, -4.52977089e-03,  4.71588017e-04],
         [-4.52977089e-03,  5.93651856e-04, -6.36371482e-05],
         [4.71588017e-04, -6.36371482e-05,  6.84130285e-06]])
    assert_almost_equal(covarianceThetaStar, exactCovarianceTheta)

    # Check other fields
    print("result=", calibrationResult)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

mesh = ot.RegularGrid(0.0, 1.0, 4)
values = [(x, 2.0 * x, x * x) for x in mesh.getValues()]
outPoint = [2.5]
interpolation = ot.P1LagrangeInterpolation(
    mesh, ot.Mesh([outPoint]), len(values[0]))
print("Interpolation=", interpolation)
print("Values at", outPoint, "=", interpolation(values))
